<?php

namespace Drupal\commerce_payrexx_integration;

use Payrexx\PayrexxException;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payrexx_integration\Plugin\Commerce\PaymentGateway\PayrexxRedirectCheckout;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Payrexx\Models\Request\Gateway;
use Payrexx\Models\Response\Transaction;
use Payrexx\Payrexx;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Service class to handle a payment response from Payrexx in Commerce.
 */
class PaymentResponseService extends TransactionHandlerService {

  /**
   * The Payrexx redirect checkout payment gateway plugin.
   *
   * @var \Drupal\commerce_payrexx_integration\Plugin\Commerce\PaymentGateway\PayrexxRedirectCheckout
   */
  protected $redirectCheckout;

  /**
   * The EntityTypeManager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * The logger channel for this module.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Constructor for PaymentResponseService.
   */
  public function __construct(PayrexxRedirectCheckout $redirectCheckout, EntityTypeManagerInterface $entityTypeManager, EventDispatcherInterface $eventDispatcher, LoggerChannelInterface $loggerChannel) {
    parent::__construct($entityTypeManager, $eventDispatcher, $loggerChannel);
    $this->redirectCheckout = $redirectCheckout;
    $this->entityTypeManager = $entityTypeManager;
    $this->eventDispatcher = $eventDispatcher;
    $this->logger = $loggerChannel;
  }

  /**
   * Handle the "return" request (success or error).
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   A commerce order.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Request object.
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $configuration = $this->redirectCheckout->getConfiguration();

    $payrexx_instance = new Payrexx($configuration['instance_name'], $configuration['secret']);

    $gid = $order->getData('payrexx_gateway_id');

    $gateway = new Gateway();
    $gateway->setId($gid);

    try {
      $response = $payrexx_instance->getOne($gateway);
      $tid = $response->getInvoices()[0]["transactions"][0]["id"];
      $transaction = new Transaction();
      $transaction->setId($tid);
      $transaction_response = $payrexx_instance->getOne($transaction);
      $this->responseHandler($order, $transaction_response);
    }
    catch (PayrexxException $e) {
      print $e->getMessage();
    }
  }

}
